package com.yx.face.dao;

import com.yx.face.boot.component.tk.TKMapper;
import com.yx.face.model.entity.UserWhitelist;
import com.yx.face.model.vo.UserWhitelistVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * @description:
 * @ClassName UserWhitelistDao
 * @Author WXG
 * @Date 2021/11/5 14:18
 */
public interface UserWhitelistDao extends TKMapper<UserWhitelist> {

    UserWhitelist getByPhoneAndId(@Param("phone")String phone ,@Param("id") Integer id,@Param("adminId") Integer adminId);

    List<UserWhitelistVO> selectListBySearch(Map<String,Object> map);

    List<UserWhitelistVO> selectList(@Param("userId") Integer userId);

    UserWhitelistVO selectListOne(@Param("id") Integer id);

    Long setUserId(@Param("id") Integer id);

    List<Long> selectId(@Param("userIds") List<Long> userIds, @Param("adminId") Integer adminId);

    Long setUserOne(@Param("id") Integer id, @Param("adminId") Integer adminId);

    UserWhitelist selectByIdNumber(@Param("idNumber") String idNumber);

    UserWhitelist selectByPhone(@Param("phone")String phone);

    UserWhitelist getByIdNubmer(@Param("idNumber") String idNumber);

}
